
package org.codehaus.mojo.codenarc
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.ArrayList

import org.apache.maven.reporting.AbstractMavenReport
import org.apache.maven.project.MavenProject
import org.apache.maven.artifact.Artifact
import org.apache.maven.artifact.factory.ArtifactFactory
import org.apache.maven.artifact.repository.ArtifactRepository
import org.apache.maven.artifact.resolver.ArtifactResolver
import org.apache.maven.doxia.siterenderer.Renderer
import org.apache.maven.doxia.tools.SiteTool
import org.codehaus.plexus.resource.ResourceManager
import org.codehaus.plexus.resource.loader.FileResourceLoader

import groovy.xml.NamespaceBuilder

/**
 * Create a CodeNarc Report.
 *
 * @goal codenarc
 * @phase site
 * @requiresDependencyResolution compile
 * @requiresProject
 * 
 * @author <a href="mailto:gleclaire@codehaus.org">Garvin LeClaire</a>
 * @version $Id: CodeNarcMojo.groovy gleclaire $
 */
class CodeNarcMojo extends AbstractMavenReport {
	/**
	 * The name of the Plug-In.
	 *
	 */
	static final String PLUGIN_NAME = "codenarc"

	/**
	 * The name of the property resource bundle (Filesystem).
	 *
	 */
	static final String BUNDLE_NAME = "codenarc"

	/**
	 * The key to get the name of the Plug-In from the bundle.
	 *
	 */
	static final String NAME_KEY = "report.codenarc.name"

	/**
	 * The key to get the description of the Plug-In from the bundle.
	 *
	 */
	static final String DESCRIPTION_KEY = "report.codenarc.description"

	/**
	 * The handle for the resource bundle.
	 *
	 */
	ResourceBundle bundle

	/**
	 * Maven Project
	 *
	 * @parameter expression="${project}"
	 * @required
	 * @readonly
	 */
	MavenProject project

	/**
	 * Location where generated html will be created.
	 *
	 * @parameter default-value="${project.reporting.outputDirectory}"
	 * @required
	 */

	File outputDirectory

	/**
	 * Specifies the directory where the xml output will be generated.
	 *
	 * @parameter default-value="${project.build.directory}"
	 * @required
	 */
	File xmlOutputDirectory

	/**
	 * Doxia Site Renderer.
	 *
	 * @component
	 * @required
	 * @readonly
	 */
	Renderer siteRenderer

	/**
	 * SiteTool.
	 *
	 * @component role="org.apache.maven.doxia.tools.SiteTool"
	 * @required
	 * @readonly
	 */
	protected SiteTool siteTool

	/**
	 * @component
	 * @required
	 * @readonly
	 */
	ResourceManager resourceManager

	/**
	 * Skip entire check.
	 *
	 * @parameter expression="${codenarc.skip}" default-value="false"
	 */
	boolean skip

	/**
	* Maximum Java heap size in megabytes  (default=512).
	*
	* @parameter default-value="512"
	*/
   int maxHeap

	/**
	 * The CodeNarc rulesets to use. See the <a href="http://codenarc.sourceforge.net/codenarc-rule-index.html">CodeNarc Rule Index</a>
	 *  for a list of some included. Defaults to the "rulesets/basic.xml,rulesets/exceptions.xml,rulesets/imports.xml"
	 *
	 * @parameter expression="${codenarc.rulesetfiles}"
	 */
	String rulesetfiles = "rulesets/basic.xml,rulesets/exceptions.xml,rulesets/imports.xml"

	/**
	 * TThe comma-separated list of Ant-style file patterns specifying files that must be included.
	 *  See the <a href="http://codenarc.sourceforge.net/codenarc-command-line.html">CodeNarc Command-Line Parameters</a>
	 *  for the default behavior.
	 *
	 * @parameter expression="${codenarc.includes}"
	 */
	String includes

	/**
	 * TThe comma-separated list of Ant-style file patterns specifying files that must be excluded.
	 *  See the <a href="http://codenarc.sourceforge.net/codenarc-command-line.html">CodeNarc Command-Line Parameters</a>
	 *  for the default behavior.
	 *
	 * @parameter expression="${codenarc.excludes}"
	 */
	String excludes

	/**
	 * The CodeNarc log4j config file.
	 * Each path may be optionally prefixed by any of the valid java.net.URL prefixes, such as "file:" 
	 * (to load from a relative or absolute filesystem path), or "http:".
	 *
	 * @parameter expression="${codenarc.log4jConfigFile}"
	 */
	String log4jConfigFile

	/**
	 * Specifies the location of the source directory to be used for Checkstyle.
	 *
	 * @parameter default-value="${project.build.sourceDirectory}"
	 * @required
	 */
	File sourceDirectory

	/**
	 * Specifies the location of the test source directory to be used for
	 * Checkstyle.
	 *
	 * @parameter default-value="${project.build.testSourceDirectory}"
	 *
	 */
	File testSourceDirectory

	/**
	 * List of artifacts this plugin depends on. Used for resolving the Findbugs coreplugin.
	 *
	 * @parameter expression="${plugin.artifacts}"
	 * @required
	 * @readonly
	 */
	ArrayList pluginArtifacts

	/**
	 * Used to look up Artifacts in the remote repository.
	 * 
	 * @parameter expression="${component.org.apache.maven.artifact.factory.ArtifactFactory}"
	 * @required
	 * @readonly
	 */
	protected ArtifactFactory factory

	/**
	 * Used to look up Artifacts in the remote repository.
	 * 
	 * @parameter expression="${component.org.apache.maven.artifact.resolver.ArtifactResolver}"
	 * @required
	 * @readonly
	 */
	protected ArtifactResolver artifactResolver

	/**
	 * List of Remote Repositories used by the resolver
	 * 
	 * @parameter expression="${project.remoteArtifactRepositories}"
	 * @readonly
	 * @required
	 */
	protected List remoteRepositories

	/**
	 * Location of the local repository.
	 * 
	 * @parameter expression="${localRepository}"
	 * @readonly
	 * @required
	 */
	protected ArtifactRepository localRepository

	/**
	 * Default log4j file content if one dosn't exist.
	 *
	 */
	def log4jContents = """# Set root logger level to DEBUG and only append to the CONSOLE.
log4j.rootLogger=DEBUG, CONSOLE

log4j.appender.CONSOLE=org.apache.log4j.ConsoleAppender

# CONSOLE uses PatternLayout.
log4j.appender.CONSOLE.layout=org.apache.log4j.PatternLayout
log4j.appender.CONSOLE.layout.ConversionPattern=%d{ISO8601} %c{1} [%t] %p - %m%n
"""

	/**
	 * The CodeNarc version to use for the plugin
	 *
	 * @parameter expression="${codenarc.codenarc.verion}"
	 */
	String codeNarcVersion = '0.16.1'

	/**
	 * The log4j version to use for the plugin
	 *
	 * @parameter expression="${codenarc.log4j.verion}"
	 */
	String log4jVersion = '1.2.13'

	/**
	 * The Groovy version to use for the plugin
	 *
	 * @parameter expression="${codenarc.groovy.verion}"
	 */
	String groovyVersion = '1.7.4'


	def items = [
		[groupId:'org.codenarc', artifactId:'CodeNarc', version:'0.16.1'],
		[groupId:'log4j', artifactId:'log4j', version:'1.2.13'],
		[groupId:'org.codehaus.groovy', artifactId:'groovy-all', version:'1.7.4']
	]


	/**
	 * Executes the generation of the report.
	 *
	 * Callback from Maven Site Plugin or from AbstractMavenReport.execute() => generate().
	 *
	 * @param locale
	 *            the locale the report should be generated for
	 * @see org.apache.maven.reporting.AbstractMavenReport #executeReport(java.util.Locale)
	 *
	 */
	protected void executeReport( Locale locale ) {
		resourceManager.addSearchPath( FileResourceLoader.ID, project.getFile().getParentFile().getAbsolutePath() )
		resourceManager.addSearchPath( "url", "" )

		resourceManager.setOutputDirectory( new File( project.getBuild().getDirectory() ) )

		log.debug("resourceManager outputDirectory is " + resourceManager.outputDirectory )

		def xmlReportFileName = "${xmlOutputDirectory}/CodeNarc.xml"
		File outputFile = new File(xmlReportFileName)

		def ant = new AntBuilder()
		Artifact pomArtifact

		def codenarcPath =  ant.path(id: "codenarc.classpath") {

			items.each() { item ->

				pomArtifact = this.factory.createArtifact( item['groupId'], item['artifactId'], item['version'],"", "jar")

				artifactResolver.resolve(pomArtifact, this.remoteRepositories, this.localRepository)

				pathelement(location: pomArtifact.file)
			}
		}

		log.info( "codenarcPath => ${codenarcPath}" )

		if (!log4jConfigFile) {
			def tempLogFile = new File( "${project.getBuild().getDirectory()}/log4j.properties"  )
			tempLogFile.text = log4jContents
			log4jConfigFile = tempLogFile.toURL()
		}

		ant.java(classname: "org.codenarc.CodeNarc", classpathref: "codenarc.classpath",fork: "true", failonerror: "false", clonevm: "false", maxmemory: "${maxHeap}m") {

			log.info( "log4jConfigFile => ${log4jConfigFile}" )
			sysproperty(key: "log4j.configuration", value: "${log4jConfigFile}")

			def antBasedir="-basedir=" +this.project.getBasedir().getAbsolutePath()
			log.info( "antBasedir => ${antBasedir}")
			arg(value: antBasedir)

			def antTitle='-title="' + this.project.name + '"'
			log.info( "antTitle => ${antTitle}" )
			arg(value: antTitle)

			def antReport="-report=xml:" + xmlReportFileName
			log.info( "antReport => ${antReport}" )
			arg(value: antReport)

			def antRuleSets="-rulesetfiles=" + rulesetfiles
			log.info( "antRuleSets => ${antRuleSets}" )
			arg(value: antRuleSets)

			if (includes) {
				def antIncludes="-includes=" + includes
				log.info( "antIncludes => ${antIncludes}" )
				arg(value: antIncludes)
			}

			if (excludes) {
				def antExcludes="-includes=" + excludes
				log.info( "antExcludes => ${antExcludes}" )
				arg(value: antExcludes)
			}
		}


		if (!outputDirectory.exists()) {
			if ( !outputDirectory.mkdirs() ) {
				fail("Cannot create HTML output directory")
			}
		}

		if (outputFile.exists()) {
			log.info("Generating CodeNarc HTML")

			CodeNarcReportGenerator generator = new CodeNarcReportGenerator( getSink(), getBundle(locale), this.project.getBasedir(), siteTool)

			generator.setLog(log)

			generator.setCodeNarcResults(new XmlSlurper().parse(outputFile))

			generator.setOutputDirectory(new File(outputDirectory.getAbsolutePath()))

			generator.generateReport()
		}
	}

	/**
	 * Checks whether prerequisites for generating this report are given.
	 *
	 * @return true if report can be generated, otherwise false
	 * @see org.apache.maven.reporting.MavenReport#canGenerateReport()
	 */
	boolean canGenerateReport() {

		def canGenerate = false

		log.info("sourceDirectory is ${sourceDirectory}" )

		if ( !skip  && sourceDirectory.exists()) {
			canGenerate = true
		}

		log.debug("canGenerate is ${canGenerate}")

		return canGenerate
	}

	/**
	 * Returns the plugins description for the "generated reports" overview page.
	 *
	 * @param locale
	 *            the locale the report should be generated for
	 *
	 * @return description of the report
	 * @see org.apache.maven.reporting.MavenReport#getDescription(java.util.Locale)
	 */
	String getDescription( Locale locale ) {
		return getBundle(locale).getString(DESCRIPTION_KEY)
	}

	/**
	 * Returns the plugins name for the "generated reports" overview page and the menu.
	 *
	 * @param locale
	 *            the locale the report should be generated for
	 *
	 * @return name of the report
	 * @see org.apache.maven.reporting.MavenReport#getName(java.util.Locale)
	 */
	String getName( Locale locale ) {
		return getBundle(locale).getString(NAME_KEY)
	}

	/**
	 * Returns report output file name, without the extension.
	 *
	 * Called by AbstractMavenReport.execute() for creating the sink.
	 *
	 * @return name of the generated page
	 * @see org.apache.maven.reporting.MavenReport#getOutputName()
	 */
	String getOutputName() {
		return PLUGIN_NAME
	}

	protected MavenProject getProject() {
		return this.project
	}

	/**
	 * Returns the report output directory.
	 *
	 * Called by AbstractMavenReport.execute() for creating the sink.
	 *
	 * @return full path to the directory where the files in the site get copied to
	 * @see org.apache.maven.reporting.AbstractMavenReport#getOutputDirectory()
	 */
	protected String getOutputDirectory() {
		return outputDirectory.getAbsolutePath()
	}

	protected Renderer getSiteRenderer() {
		return this.siteRenderer
	}

	ResourceBundle getBundle(locale) {

		this.bundle = ResourceBundle.getBundle(BUNDLE_NAME, locale, CodeNarcMojo.class.getClassLoader())

		log.debug("Mojo Locale is " + this.bundle.getLocale().getLanguage())

		return bundle
	}
}
